@@ -1,4 +1,5 @@
 """SDL2 wrapper package"""
+from __future__ import absolute_import
 from .dll import get_dll_file, _bind
 from ctypes import c_int as _cint
 
@@ -13,6 +14,7 @@ from .filesystem import *
 from .gamecontroller import *
 from .gesture import *
 from .haptic import *
+from .hidapi import *
 from .hints import *
 from .joystick import *
 from .keyboard import *
@@ -68,5 +70,5 @@ SDL_QuitSubSystem = _bind("SDL_QuitSubSystem", [Uint32])
 SDL_WasInit = _bind("SDL_WasInit", [Uint32], Uint32)
 SDL_Quit = _bind("SDL_Quit")
 
-__version__ = "0.9.8"
-version_info = (0, 9, 8)
+__version__ = "0.9.10"
+version_info = (0, 9, 10)
