@@ -6,26 +6,35 @@ __all__ = [
     # Defines
     "SDL_HINT_FRAMEBUFFER_ACCELERATION", "SDL_HINT_RENDER_DRIVER",
     "SDL_HINT_RENDER_OPENGL_SHADERS", "SDL_HINT_RENDER_SCALE_QUALITY",
-    "SDL_HINT_RENDER_VSYNC", "SDL_HINT_VIDEO_X11_XVIDMODE",
+    "SDL_HINT_RENDER_VSYNC", "SDL_HINT_VIDEO_WAYLAND_ALLOW_LIBDECOR",
+    "SDL_HINT_VIDEO_X11_XVIDMODE",
     "SDL_HINT_VIDEO_X11_XINERAMA", "SDL_HINT_VIDEO_X11_XRANDR",
     "SDL_HINT_VIDEO_X11_WINDOW_VISUALID",
     "SDL_HINT_VIDEO_X11_NET_WM_PING",
     "SDL_HINT_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR",
     "SDL_HINT_VIDEO_X11_FORCE_EGL",
+    "SDL_HINT_X11_FORCE_OVERRIDE_REDIRECT",
     "SDL_HINT_GRAB_KEYBOARD", "SDL_HINT_VIDEO_MINIMIZE_ON_FOCUS_LOSS",
     "SDL_HINT_IDLE_TIMER_DISABLED", "SDL_HINT_ORIENTATIONS",
     "SDL_HINT_XINPUT_ENABLED",
     "SDL_HINT_ALLOW_TOPMOST", "SDL_HINT_JOYSTICK_ALLOW_BACKGROUND_EVENTS",
     "SDL_HINT_JOYSTICK_HIDAPI",
-    "SDL_HINT_JOYSTICK_HIDAPI_PS4", "SDL_HINT_JOYSTICK_HIDAPI_PS5",
-    "SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE",
+    "SDL_HINT_JOYSTICK_HIDAPI_PS4", "SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE",
+    "SDL_HINT_JOYSTICK_HIDAPI_PS5", "SDL_HINT_JOYSTICK_HIDAPI_PS5_RUMBLE",
+    "SDL_HINT_JOYSTICK_HIDAPI_PS5_PLAYER_LED",
+    "SDL_HINT_JOYSTICK_HIDAPI_STADIA",
     "SDL_HINT_JOYSTICK_HIDAPI_STEAM",
     "SDL_HINT_JOYSTICK_HIDAPI_SWITCH",
+    "SDL_HINT_JOYSTICK_HIDAPI_SWITCH_HOME_LED",
+    "SDL_HINT_JOYSTICK_HIDAPI_JOY_CONS",
     "SDL_HINT_JOYSTICK_HIDAPI_XBOX",
     "SDL_HINT_JOYSTICK_HIDAPI_CORRELATE_XINPUT",
     "SDL_HINT_JOYSTICK_HIDAPI_GAMECUBE",
+    "SDL_HINT_JOYSTICK_HIDAPI_LUNA",
     "SDL_HINT_ENABLE_STEAM_CONTROLLERS",
+    "SDL_HINT_JOYSTICK_RAWINPUT_CORRELATE_XINPUT",
     "SDL_HINT_JOYSTICK_RAWINPUT", "SDL_HINT_JOYSTICK_THREAD",
+    "SDL_HINT_JOYSTICK_DEVICE", "SDL_HINT_LINUX_JOYSTICK_CLASSIC",
     "SDL_HINT_LINUX_JOYSTICK_DEADZONES", "SDL_HINT_AUTO_UPDATE_JOYSTICKS",
     "SDL_HINT_AUTO_UPDATE_SENSORS",
     "SDL_HINT_VIDEO_HIGHDPI_DISABLED",
@@ -45,11 +54,12 @@ __all__ = [
     "SDL_HINT_WINRT_PRIVACY_POLICY_LABEL",
     "SDL_HINT_WINRT_HANDLE_BACK_BUTTON",
     "SDL_HINT_WINDOW_FRAME_USABLE_WHILE_CURSOR_HIDDEN",
+    "SDL_HINT_WINDOW_NO_ACTIVATION_WHEN_SHOWN",
     "SDL_HINT_WINDOWS_ENABLE_MESSAGELOOP",
     "SDL_HINT_ANDROID_APK_EXPANSION_MAIN_FILE_VERSION",
     "SDL_HINT_ANDROID_APK_EXPANSION_PATCH_FILE_VERSION",
     "SDL_HINT_XINPUT_USE_OLD_JOYSTICK_MAPPING",
-    "SDL_HINT_IME_INTERNAL_EDITING",
+    "SDL_HINT_IME_INTERNAL_EDITING", "SDL_HINT_IME_SHOW_UI",
     "SDL_HINT_EMSCRIPTEN_KEYBOARD_ELEMENT", "SDL_HINT_EMSCRIPTEN_ASYNCIFY",
     "SDL_HINT_NO_SIGNAL_HANDLERS",
     "SDL_HINT_ANDROID_SEPARATE_MOUSE_AND_TOUCH",
@@ -65,7 +75,9 @@ __all__ = [
     "SDL_HINT_APPLE_TV_REMOTE_ALLOW_ROTATION",
     "SDL_HINT_BMP_SAVE_LEGACY_FORMAT",
     "SDL_HINT_WINDOWS_DISABLE_THREAD_NAMING",
-    "SDL_HINT_RPI_VIDEO_LAYER",
+    "SDL_HINT_WINDOWS_FORCE_MUTEX_CRITICAL_SECTIONS",
+    "SDL_HINT_WINDOWS_FORCE_SEMAPHORE_KERNEL", "SDL_HINT_WINDOWS_USE_D3D9EX",
+    "SDL_HINT_RPI_VIDEO_LAYER", "SDL_HINT_SCREENSAVER_INHIBIT_ACTIVITY_NAME",
     "SDL_HINT_QTWAYLAND_CONTENT_ORIENTATION",
     "SDL_HINT_QTWAYLAND_WINDOW_FLAGS",
     "SDL_HINT_MOUSE_NORMAL_SPEED_SCALE",
@@ -83,11 +95,16 @@ __all__ = [
     "SDL_HINT_WINDOWS_INTRESOURCE_ICON",
     "SDL_HINT_WINDOWS_INTRESOURCE_ICON_SMALL", "SDL_HINT_AUDIO_CATEGORY",
     "SDL_HINT_AUDIO_DEVICE_APP_NAME", "SDL_HINT_AUDIO_DEVICE_STREAM_NAME",
-    "SDL_HINT_PREFERRED_LOCALES",
+    "SDL_HINT_AUDIO_DEVICE_STREAM_ROLE", "SDL_HINT_AUDIO_INCLUDE_MONITORS",
+    "SDL_HINT_ALLOW_ALT_TAB_WHILE_GRABBED",
+    "SDL_HINT_PREFERRED_LOCALES", "SDL_HINT_APP_NAME",
     "SDL_HINT_RENDER_BATCHING", "SDL_HINT_EVENT_LOGGING",
+    "SDL_HINT_POLL_SENTINEL",
     "SDL_HINT_WAVE_RIFF_CHUNK_SIZE", "SDL_HINT_WAVE_TRUNCATION",
     "SDL_HINT_WAVE_FACT_CHUNK", "SDL_HINT_DISPLAY_USABLE_BOUNDS",
-    "SDL_HINT_VIDEO_DOUBLE_BUFFER", "SDL_HINT_RETURN_KEY_HIDES_IME",
+    "SDL_HINT_VIDEO_DOUBLE_BUFFER", "SDL_HINT_VIDEO_EGL_ALLOW_TRANSPARENCY",
+    "SDL_HINT_KMSDRM_REQUIRE_DRM_MASTER",
+    "SDL_HINT_RETURN_KEY_HIDES_IME",
     "SDL_HINT_TV_REMOTE_AS_JOYSTICK", "SDL_HINT_IOS_HIDE_HOME_INDICATOR",
 
     # Enums
@@ -114,6 +131,7 @@ SDL_HINT_RENDER_SCALE_QUALITY = b"SDL_RENDER_SCALE_QUALITY"
 SDL_HINT_RENDER_VSYNC = b"SDL_RENDER_VSYNC"
 SDL_HINT_VIDEO_ALLOW_SCREENSAVER = b"SDL_VIDEO_ALLOW_SCREENSAVER"
 SDL_HINT_VIDEO_EXTERNAL_CONTEXT = b"SDL_VIDEO_EXTERNAL_CONTEXT"
+SDL_HINT_VIDEO_WAYLAND_ALLOW_LIBDECOR = b"SDL_VIDEO_WAYLAND_ALLOW_LIBDECOR"
 SDL_HINT_VIDEO_X11_XVIDMODE = b"SDL_VIDEO_X11_XVIDMODE"
 SDL_HINT_VIDEO_X11_XINERAMA = b"SDL_VIDEO_X11_XINERAMA"
 SDL_HINT_VIDEO_X11_XRANDR = b"SDL_VIDEO_X11_XRANDR"
@@ -121,7 +139,9 @@ SDL_HINT_VIDEO_X11_WINDOW_VISUALID = b"SDL_VIDEO_X11_WINDOW_VISUALID"
 SDL_HINT_VIDEO_X11_NET_WM_PING = b"SDL_VIDEO_X11_NET_WM_PING"
 SDL_HINT_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR = b"SDL_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR"
 SDL_HINT_VIDEO_X11_FORCE_EGL = b"SDL_VIDEO_X11_FORCE_EGL"
+SDL_HINT_X11_FORCE_OVERRIDE_REDIRECT = b"SDL_X11_FORCE_OVERRIDE_REDIRECT"
 SDL_HINT_WINDOW_FRAME_USABLE_WHILE_CURSOR_HIDDEN = b"SDL_WINDOW_FRAME_USABLE_WHILE_CURSOR_HIDDEN"
+SDL_HINT_WINDOW_NO_ACTIVATION_WHEN_SHOWN = b"SDL_WINDOW_NO_ACTIVATION_WHEN_SHOWN"
 
 SDL_HINT_WINDOWS_INTRESOURCE_ICON = b"SDL_WINDOWS_INTRESOURCE_ICON"
 SDL_HINT_WINDOWS_INTRESOURCE_ICON_SMALL = b"SDL_WINDOWS_INTRESOURCE_ICON_SMALL"
@@ -158,16 +178,25 @@ SDL_HINT_GAMECONTROLLER_USE_BUTTON_LABELS = b"SDL_GAMECONTROLLER_USE_BUTTON_LABE
 SDL_HINT_JOYSTICK_ALLOW_BACKGROUND_EVENTS = b"SDL_JOYSTICK_ALLOW_BACKGROUND_EVENTS"
 SDL_HINT_JOYSTICK_HIDAPI = b"SDL_JOYSTICK_HIDAPI"
 SDL_HINT_JOYSTICK_HIDAPI_PS4 = b"SDL_JOYSTICK_HIDAPI_PS4"
-SDL_HINT_JOYSTICK_HIDAPI_PS5 = b"SDL_JOYSTICK_HIDAPI_PS5"
 SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE = b"SDL_JOYSTICK_HIDAPI_PS4_RUMBLE"
+SDL_HINT_JOYSTICK_HIDAPI_PS5 = b"SDL_JOYSTICK_HIDAPI_PS5"
+SDL_HINT_JOYSTICK_HIDAPI_PS5_RUMBLE = b"SDL_JOYSTICK_HIDAPI_PS5_RUMBLE"
+SDL_HINT_JOYSTICK_HIDAPI_PS5_PLAYER_LED = b"SDL_JOYSTICK_HIDAPI_PS5_PLAYER_LED"
+SDL_HINT_JOYSTICK_HIDAPI_STADIA = b"SDL_JOYSTICK_HIDAPI_STADIA"
 SDL_HINT_JOYSTICK_HIDAPI_STEAM = b"SDL_JOYSTICK_HIDAPI_STEAM"
 SDL_HINT_JOYSTICK_HIDAPI_SWITCH = b"SDL_JOYSTICK_HIDAPI_SWITCH"
+SDL_HINT_JOYSTICK_HIDAPI_SWITCH_HOME_LED = b"SDL_JOYSTICK_HIDAPI_SWITCH_HOME_LED"
+SDL_HINT_JOYSTICK_HIDAPI_JOY_CONS = b"SDL_JOYSTICK_HIDAPI_JOY_CONS"
 SDL_HINT_JOYSTICK_HIDAPI_XBOX = b"SDL_JOYSTICK_HIDAPI_XBOX"
 SDL_HINT_JOYSTICK_HIDAPI_CORRELATE_XINPUT = b"SDL_JOYSTICK_HIDAPI_CORRELATE_XINPUT"
 SDL_HINT_JOYSTICK_HIDAPI_GAMECUBE = b"SDL_JOYSTICK_HIDAPI_GAMECUBE"
+SDL_HINT_JOYSTICK_HIDAPI_LUNA = b"SDL_JOYSTICK_HIDAPI_LUNA"
 SDL_HINT_ENABLE_STEAM_CONTROLLERS = b"SDL_ENABLE_STEAM_CONTROLLERS"
 SDL_HINT_JOYSTICK_RAWINPUT = b"SDL_JOYSTICK_RAWINPUT"
+SDL_HINT_JOYSTICK_RAWINPUT_CORRELATE_XINPUT = b"SDL_JOYSTICK_RAWINPUT_CORRELATE_XINPUT"
 SDL_HINT_JOYSTICK_THREAD = b"SDL_JOYSTICK_THREAD"
+SDL_HINT_JOYSTICK_DEVICE = b"SDL_JOYSTICK_DEVICE"
+SDL_HINT_LINUX_JOYSTICK_CLASSIC = b"SDL_LINUX_JOYSTICK_CLASSIC"
 SDL_HINT_LINUX_JOYSTICK_DEADZONES = b"SDL_LINUX_JOYSTICK_DEADZONES"
 SDL_HINT_AUTO_UPDATE_JOYSTICKS = b"SDL_AUTO_UPDATE_JOYSTICKS"
 SDL_HINT_AUTO_UPDATE_SENSORS = b"SDL_AUTO_UPDATE_SENSORS"
@@ -193,6 +222,7 @@ SDL_HINT_MAC_BACKGROUND_APP = b"SDL_MAC_BACKGROUND_APP"
 SDL_HINT_ANDROID_APK_EXPANSION_MAIN_FILE_VERSION = b"SDL_ANDROID_APK_EXPANSION_MAIN_FILE_VERSION"
 SDL_HINT_ANDROID_APK_EXPANSION_PATCH_FILE_VERSION = b"SDL_ANDROID_APK_EXPANSION_PATCH_FILE_VERSION"
 SDL_HINT_IME_INTERNAL_EDITING = b"SDL_IME_INTERNAL_EDITING"
+SDL_HINT_IME_SHOW_UI = b"SDL_IME_SHOW_UI"
 SDL_HINT_ANDROID_SEPARATE_MOUSE_AND_TOUCH = b"SDL_ANDROID_SEPARATE_MOUSE_AND_TOUCH" # removed in 2.0.10
 SDL_HINT_ANDROID_TRAP_BACK_BUTTON = b"SDL_ANDROID_TRAP_BACK_BUTTON"
 SDL_HINT_ANDROID_BLOCK_ON_PAUSE = b"SDL_ANDROID_BLOCK_ON_PAUSE"
@@ -205,18 +235,30 @@ SDL_HINT_NO_SIGNAL_HANDLERS = b"SDL_NO_SIGNAL_HANDLERS"
 SDL_HINT_WINDOWS_NO_CLOSE_ON_ALT_F4 = b"SDL_WINDOWS_NO_CLOSE_ON_ALT_F4"
 SDL_HINT_BMP_SAVE_LEGACY_FORMAT = b"SDL_BMP_SAVE_LEGACY_FORMAT"
 SDL_HINT_WINDOWS_DISABLE_THREAD_NAMING = b"SDL_WINDOWS_DISABLE_THREAD_NAMING"
+SDL_HINT_WINDOWS_FORCE_MUTEX_CRITICAL_SECTIONS = b"SDL_WINDOWS_FORCE_MUTEX_CRITICAL_SECTIONS"
+SDL_HINT_WINDOWS_FORCE_SEMAPHORE_KERNEL = b"SDL_WINDOWS_FORCE_SEMAPHORE_KERNEL"
+SDL_HINT_WINDOWS_USE_D3D9EX = b"SDL_WINDOWS_USE_D3D9EX"
 
 SDL_HINT_RPI_VIDEO_LAYER = b"SDL_RPI_VIDEO_LAYER"
+SDL_HINT_SCREENSAVER_INHIBIT_ACTIVITY_NAME = b"SDL_SCREENSAVER_INHIBIT_ACTIVITY_NAME"
 SDL_HINT_VIDEO_DOUBLE_BUFFER = b"SDL_VIDEO_DOUBLE_BUFFER"
+SDL_HINT_VIDEO_EGL_ALLOW_TRANSPARENCY = b"SDL_VIDEO_EGL_ALLOW_TRANSPARENCY"
+SDL_HINT_KMSDRM_REQUIRE_DRM_MASTER = b"SDL_KMSDRM_REQUIRE_DRM_MASTER"
 SDL_HINT_OPENGL_ES_DRIVER = b"SDL_OPENGL_ES_DRIVER"
 
 SDL_HINT_AUDIO_RESAMPLING_MODE = b"SDL_AUDIO_RESAMPLING_MODE"
 SDL_HINT_AUDIO_CATEGORY = b"SDL_AUDIO_CATEGORY"
 SDL_HINT_AUDIO_DEVICE_APP_NAME = b"SDL_AUDIO_DEVICE_APP_NAME"
 SDL_HINT_AUDIO_DEVICE_STREAM_NAME = b"SDL_AUDIO_DEVICE_STREAM_NAME"
+SDL_HINT_AUDIO_DEVICE_STREAM_ROLE = b"SDL_AUDIO_DEVICE_STREAM_ROLE"
+SDL_HINT_AUDIO_INCLUDE_MONITORS = b"SDL_AUDIO_INCLUDE_MONITORS"
+
+SDL_HINT_ALLOW_ALT_TAB_WHILE_GRABBED = b"SDL_ALLOW_ALT_TAB_WHILE_GRABBED"
+SDL_HINT_APP_NAME = b"SDL_APP_NAME"
 SDL_HINT_PREFERRED_LOCALES = b"SDL_PREFERRED_LOCALES"
 SDL_HINT_RENDER_BATCHING  = b"SDL_RENDER_BATCHING"
 SDL_HINT_EVENT_LOGGING = b"SDL_EVENT_LOGGING"
+SDL_HINT_POLL_SENTINEL = b"SDL_POLL_SENTINEL"
 SDL_HINT_WAVE_RIFF_CHUNK_SIZE = b"SDL_WAVE_RIFF_CHUNK_SIZE"
 SDL_HINT_WAVE_TRUNCATION = b"SDL_WAVE_TRUNCATION"
 SDL_HINT_WAVE_FACT_CHUNK = b"SDL_WAVE_FACT_CHUNK"
@@ -233,7 +275,7 @@ SDL_HINT_OVERRIDE = 2
 SDL_SetHintWithPriority = _bind("SDL_SetHintWithPriority", [c_char_p, c_char_p, SDL_HintPriority], SDL_bool)
 SDL_SetHint = _bind("SDL_SetHint", [c_char_p, c_char_p], SDL_bool)
 SDL_GetHint = _bind("SDL_GetHint", [c_char_p], c_char_p)
-SDL_GetHintBoolean = _bind("SDL_GetHintBoolean", [c_char_p, SDL_bool], SDL_bool)
+SDL_GetHintBoolean = _bind("SDL_GetHintBoolean", [c_char_p, SDL_bool], SDL_bool, added='2.0.5')
 SDL_ClearHints = _bind("SDL_ClearHints")
 SDL_HintCallback = CFUNCTYPE(None, c_void_p, c_char_p, c_char_p, c_char_p)
 SDL_AddHintCallback = _bind("SDL_AddHintCallback", [c_char_p, SDL_HintCallback, c_void_p])
